/*
 * Copyright (c) 1997-2003 Erez Zadok
 * Copyright (c) 2001-2003 Stony Brook University
 * Copyright (c) 1997-2000 Columbia University
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package, or get one from ftp://ftp.filesystems.org/pub/fist/COPYING.
 *
 * This Copyright notice must be kept intact and distributed with all
 * fistgen sources INCLUDING sources generated by fistgen.
 */
/*
 *  $Id: logeventfs.h,v 1.20 2002/12/27 20:19:06 ezk Exp $
 */

#ifndef __LOGEVENTFS_H_
#define __LOGEVENTFS_H_

#ifdef __KERNEL__

/*
 * STRUCTURES:
 */


/* Turn this on to include code that counts writes and writes in the middle */
/* #undef FIST_COUNT_WRITES */

/* fist file systems superblock magic */
#define LOGEVENTFS_SUPER_MAGIC 0xf15f


/* logeventfs inode data in memory */
struct logeventfs_inode_info {
    inode_t *wii_inode;
};

/* logeventfs dentry data in memory */
struct logeventfs_dentry_info {
    dentry_t *wdi_dentry;
};


/* logeventfs super-block data in memory */
struct logeventfs_sb_info {
    super_block_t *wsi_sb;
    struct vfsmount *hidden_mnt;
    ;
};

/* file private data. */
struct logeventfs_file_info {
    struct file *wfi_file;
};

/*
 * MACROS:
 */

// File TO Private Data
#define ftopd(file) ((struct logeventfs_file_info *)((file)->private_data))
// File TO Hidden File
#define ftohf(file) ((ftopd(file))->wfi_file)
// Inode TO Private Data
#define itopd(ino) ((struct logeventfs_inode_info *)(ino)->u.generic_ip)
// Inode TO Hidden Inode
#define itohi(ino) (itopd(ino)->wii_inode)
#define vnode2lower itohi
// Superblock TO Private Data
#define stopd(super) ((struct logeventfs_sb_info *)(super)->u.generic_sbp)
#define vfs2priv stopd
// Superblock TO Hidden Superblock
#define stohs(super) (stopd(super)->wsi_sb)
// Dentry TO Hidden Dentry (and private data --- for the future)
#define dtopd(dentry) ((struct logeventfs_dentry_info *)(dentry)->d_fsdata)
#define dtohd(dent) (dtopd(dent)->wdi_dentry)
#define dir2lower dtohd
/* VMArea TO Hidden VMArea */
#define vmatohvma(vma) ((vm_area_t *)((vma)->vm_private_data))

#define sbt(sb) ((sb)->s_type->name)

/* macros to simplify non-SCA code */
#define MALLOC_PAGE_POINTERS(hidden_pages, num_hidden_pages)
#define MALLOC_PAGEDATA_POINTERS(hidden_pages_data, num_hidden_pages)
#define FREE_PAGE_POINTERS(hidden_pages, num)
#define FREE_PAGEDATA_POINTERS(hidden_pages_data, num)
#define FOR_EACH_PAGE
#define CURRENT_HIDDEN_PAGE hidden_page
#define CURRENT_HIDDEN_PAGEDATA hidden_page_data
#define CURRENT_HIDDEN_PAGEINDEX page->index



/*
 * EXTERNALS:
 */
extern struct file_operations logeventfs_main_fops;
extern struct file_operations logeventfs_dir_fops;
extern struct inode_operations logeventfs_main_iops;
extern struct inode_operations logeventfs_dir_iops;
extern struct inode_operations logeventfs_symlink_iops;
extern struct super_operations logeventfs_sops;
extern struct dentry_operations logeventfs_dops;
extern struct vm_operations_struct logeventfs_shared_vmops;
extern struct vm_operations_struct logeventfs_private_vmops;
extern struct address_space_operations logeventfs_aops;

extern int logeventfs_interpose(dentry_t *hidden_dentry, dentry_t *this_dentry, super_block_t *sb, int flag);
#if defined(FIST_FILTER_DATA) || defined(FIST_FILTER_SCA)
extern page_t *logeventfs_get1page(file_t *file, int index);
extern int logeventfs_fill_zeros(file_t *file, page_t *page, unsigned from);
#endif /* FIST_FILTER_DATA || FIST_FILTER_SCA */

# define logeventfs_hidden_dentry(dentry) dtohd(dentry)




#define copy_inode_size(dst, src) \
    dst->i_size = src->i_size; \
    dst->i_blocks = src->i_blocks;

static inline void
fist_copy_attr_atime(inode_t *dest, const inode_t *src)
{
    ASSERT(dest != NULL);
    ASSERT(src != NULL);
    dest->i_atime = src->i_atime;
}
static inline void
fist_copy_attr_times(inode_t *dest, const inode_t *src)
{
    ASSERT(dest != NULL);
    ASSERT(src != NULL);
    dest->i_atime = src->i_atime;
    dest->i_mtime = src->i_mtime;
    dest->i_ctime = src->i_ctime;
}
static inline void
fist_copy_attr_timesizes(inode_t *dest, const inode_t *src)
{
    ASSERT(dest != NULL);
    ASSERT(src != NULL);
    dest->i_atime = src->i_atime;
    dest->i_mtime = src->i_mtime;
    dest->i_ctime = src->i_ctime;
    copy_inode_size(dest, src);
}
static inline void
fist_copy_attr_all(inode_t *dest, const inode_t *src)
{
    print_entry_location();
    ASSERT(dest != NULL);
    ASSERT(src != NULL);
    dest->i_mode = src->i_mode;
    dest->i_nlink = src->i_nlink;
    dest->i_uid = src->i_uid;
    dest->i_gid = src->i_gid;
    dest->i_rdev = src->i_rdev;
    dest->i_atime = src->i_atime;
    dest->i_mtime = src->i_mtime;
    dest->i_ctime = src->i_ctime;
    dest->i_blksize = src->i_blksize;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,12)
    dest->i_blkbits = src->i_blkbits;
#endif /* linux 2.4.12 and newer */
    copy_inode_size(dest, src);

    dest->i_attr_flags = src->i_attr_flags;
    print_exit_location();
}

/* auto-generated extern definitions */


#endif /* __KERNEL__ */

/*
 * Definitions for user and kernel code
 */

/* ioctls */


#endif	/* not __LOGEVENTFS_H_ */

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
